/*******************************************************************************
 *  PROJECT: GNOME Colorscheme
 *
 *  AUTHOR: Jonathon Jongsma
 *
 *  Copyright (c) 2005 Jonathon Jongsma
 *
 *  License:
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the 
 *    Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 *    Boston, MA  02111-1307  USA
 *
 *******************************************************************************/

#include <memory>   // for auto_ptr
#include <vector>
#include <cstdlib>  // for srand(), rand()
#include <ctime>    // for time()

#include <gtkmm/stock.h>
#include <gdkmm/pixbuf.h>
#include <gtkmm/iconfactory.h>
#include <gtkmm/toolbar.h>
#include <libgnomevfsmm.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gcs-mainwindow.h"
#include "gcs-debug.h"
#include "gcs-i18n.h"
#include "gcs-history.h"
#include "core/gcs-scheme.h"
#include "widgets/gcs-schemebox.h"
#include "widgets/gcs-schemeselector.h"
#include "widgets/gcs-bookmarklist.h"
#include "widgets/gcs-colorswatch.h"
#include "widgets/gcs-paletteview.h"
#include "dialogs/gcs-about-window.h"

#define NO_SIZE_REQUEST (-1)

namespace gcs
{
    MainWindow* MainWindow::m_instance = NULL;
    const gint MainWindow::WINDOW_HEIGHT = 500;
    const gint MainWindow::WINDOW_WIDTH = 650;
    const gint MainWindow::FAVORITES_WIDTH = 130;
    const Glib::ustring MainWindow::FAVORITES_FILE = "bookmarks.gpl";
    const Glib::ustring MainWindow::GCS_CONF_DIR = ".colorscheme";

    MainWindow& MainWindow::Instance(void)
    {
        if (m_instance)
        {
            return *m_instance;
        }
        else
        {
            m_instance = new MainWindow();
            return *m_instance;
        }
    }


    MainWindow::MainWindow(void)
        : m_pLayout(Gtk::manage(new Gtk::VBox())),
        m_pInnerLayout(Gtk::manage(new Gtk::HBox())),
        m_pScrollBox(Gtk::manage(new Gtk::ScrolledWindow())),
        m_pSchemeColumn(Gtk::manage(new Gtk::VBox())),
        m_pBookmarkColumn(Gtk::manage(new Gtk::VBox())),
        m_pPane(Gtk::manage(new Gtk::HPaned())),
        m_pControls(Gtk::manage(new Gtk::HBox())),
        m_pSchemeBox(Gtk::manage(new Widgets::SchemeBox())),
        m_PaletteView(Gtk::manage(new Widgets::PaletteView(
                        Gnome::Vfs::Uri::create(Gnome::Vfs::get_uri_from_local_path(
                                COLORSCHEME_PALETTEDIR))->append_path("Web.gpl")))),
        m_pStatusbar(Gtk::manage(new Gtk::Statusbar())),
        m_pHistory(new HistoryNavigation<tHexString>()),
        m_pMainMenu(0),  // will be assigned in the constructor
        m_bookmarks_file(Gnome::Vfs::Uri::create(Gnome::Vfs::get_uri_from_local_path(
                        Glib::get_home_dir()))->append_path(GCS_CONF_DIR)->append_path(FAVORITES_FILE))
    {
        m_pBookmarkList = Gtk::manage(new Widgets::BookmarkList(m_bookmarks_file));
        init_actions();
        init_ui();

        // seed the random number generator for generating random color schemes
        srand(time(NULL));


        /* The menu bar across the top of the window */
        m_pMainMenu = static_cast<Gtk::Menu *>(
                m_refUIManager->get_widget("/MainMenu"));
        if (m_pMainMenu != NULL)
        {
            m_pLayout->pack_start(*m_pMainMenu, Gtk::PACK_SHRINK);
        }
        debug("Added main menu");

        m_pTweakBar = static_cast<Gtk::Toolbar *>(
                m_refUIManager->get_widget("/TweakBar"));
        if (m_pTweakBar != NULL)
        {
            m_pTweakBar->set_toolbar_style(Gtk::TOOLBAR_BOTH_HORIZ);
            m_pLayout->pack_start(*m_pTweakBar, Gtk::PACK_SHRINK, 0);
        }

        if (m_pPane != NULL)
        {
            m_pLayout->pack_start(*m_pPane);
            gint w, h;
            get_size(w, h);
            m_pPane->set_position(w - FAVORITES_WIDTH);
        }

        if (m_pInnerLayout != NULL)
        {
            m_pPane->pack1(*m_pInnerLayout);
        }

        if (m_pSchemeColumn != NULL)
        {
            m_pInnerLayout->pack_start(*m_pSchemeColumn);
        }
        if (m_pBookmarkColumn != NULL)
        {
            m_pPane->pack2(*m_pBookmarkColumn);
        }

        // Set up the SchemeBox
        if (m_pSchemeBox != NULL)
        {
            Gtk::Menu * pSwatchPopup = static_cast<Gtk::Menu *>(
                    m_refUIManager->get_widget("/ColorSwatchPopup"));
            debug("popup widget: ", pSwatchPopup);
            m_pSchemeColumn->pack_start(*m_pSchemeBox, Gtk::PACK_EXPAND_WIDGET, 0);
            m_pSchemeBox->signal_color_selected().connect(
                    sigc::mem_fun(*this, &MainWindow::on_schemebox_color_selected));
        }

        // set up the main controls
        if (m_pControls != NULL)
        {
            m_pSchemeColumn->pack_start(*m_pControls, Gtk::PACK_SHRINK);
        }
        debug("Added Controls box");

        m_pColorButton = Gtk::manage(new Gtk::ColorButton);
        m_pColorButton->signal_color_set().connect(sigc::mem_fun(*this,
                    &MainWindow::on_color_changed));
        debug("created Color Button");
        m_pSchemeSelector = Gtk::manage(new Widgets::SchemeSelector);
        m_pSchemeSelector->signal_changed().connect(sigc::mem_fun(*this,
                    &MainWindow::on_schemetype_changed));
        debug("Created Scheme Selector");
        m_pControls->pack_start(*m_pColorButton, Gtk::PACK_SHRINK);
        m_pControls->pack_start(*m_pSchemeSelector, Gtk::PACK_EXPAND_WIDGET);
        m_pControls->set_spacing(6);
        m_pControls->set_border_width(6);

        if (m_PaletteView != NULL)
        {
            m_pSchemeColumn->pack_start(*m_PaletteView, Gtk::PACK_SHRINK, 0);
            m_PaletteView->set_border_width(6);
            m_PaletteView->signal_color_selected().connect(
                    sigc::mem_fun(*this, &MainWindow::on_schemebox_color_selected));
            m_PaletteView->set_expanded();
        }


        if (m_pBookmarkList != NULL)
        {
            m_pBookmarkList->set_size_request(FAVORITES_WIDTH, NO_SIZE_REQUEST);
            m_pScrollBox->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
            m_pBookmarkColumn->pack_start(*m_pScrollBox);
            m_pScrollBox->add(*m_pBookmarkList);
        }
        m_pBookmarkList->get_selection()->signal_changed().connect(
                sigc::mem_fun(*this, &MainWindow::on_bookmarks_selection_changed));
        
        m_pBookmarkBar = dynamic_cast<Gtk::Toolbar *>(m_refUIManager->get_widget("/BookmarkBar"));
        if (m_pBookmarkBar != NULL)
        {
            m_pBookmarkColumn->pack_start(*m_pBookmarkBar, Gtk::PACK_SHRINK);
            m_pBookmarkBar->set_toolbar_style(Gtk::TOOLBAR_BOTH_HORIZ);
        }

        m_pStatusbar->push(_("Choose a Color and a Scheme Type"), 1);
        if (m_pStatusbar != NULL)
        {
            m_pLayout->pack_start(*m_pStatusbar, Gtk::PACK_SHRINK);
        }

        m_pAbout = new Dialogs::AboutWindow();

        add(*m_pLayout);
        m_pLayout->show_all();
        set_position(Gtk::WIN_POS_CENTER);
        try
        {
            char *icon_file = COLORSCHEME_ICONDIR "/colorscheme-icon.png";
            debug("icon: ", icon_file);
            set_icon(Gdk::Pixbuf::create_from_file(icon_file
                        )
                    );
        }
        catch(Glib::Error& ex)
        {
            std::cerr << "Icon Image not found" << std::endl
                << ex.what() << std::endl;
        }

        //set_default_size(WINDOW_WIDTH, WINDOW_HEIGHT);
#ifdef HAVE_CONFIG_H
        set_title(PACKAGE_NAME);
#endif
    }

    MainWindow::~MainWindow(void)
    {
        delete m_pAbout;
        //delete m_pHistory;
        debug("MainWindow DELETED!");
    }


    void MainWindow::init_actions(void)
    {
        // Register stock icons for GNOME Colorscheme
        Glib::RefPtr<Gtk::IconFactory> iconFactory = Gtk::IconFactory::create();
        iconFactory->add_default();

        // the icon for lightening a scheme
        try
        {
        Glib::RefPtr<Gdk::Pixbuf> refLighten = Gdk::Pixbuf::create_from_file(
                COLORSCHEME_ICONDIR "/lighten.png");
        std::auto_ptr<Gtk::StockID> pLightenID(new Gtk::StockID("gcs-lighten"));
        std::auto_ptr<Gtk::IconSet> pLighten(new Gtk::IconSet(refLighten));

        // the icon for darkening a scheme
        Glib::RefPtr<Gdk::Pixbuf> refDarken = Gdk::Pixbuf::create_from_file(
                COLORSCHEME_ICONDIR "/darken.png");
        std::auto_ptr<Gtk::StockID> pDarkenID(new Gtk::StockID("gcs-darken"));
        std::auto_ptr<Gtk::IconSet> pDarken(new Gtk::IconSet(refDarken));

        // the icon for increasing a scheme's saturation
        Glib::RefPtr<Gdk::Pixbuf> refSat = Gdk::Pixbuf::create_from_file(
                COLORSCHEME_ICONDIR "/saturate.png");
        std::auto_ptr<Gtk::StockID> pSatID(new Gtk::StockID("gcs-saturate"));
        std::auto_ptr<Gtk::IconSet> pSat(new Gtk::IconSet(refSat));

        // the icon for desaturating a scheme
        Glib::RefPtr<Gdk::Pixbuf> refDesat = Gdk::Pixbuf::create_from_file(
                COLORSCHEME_ICONDIR "/desaturate.png");
        std::auto_ptr<Gtk::StockID> pDesatID(new Gtk::StockID("gcs-desaturate"));
        std::auto_ptr<Gtk::IconSet> pDesat(new Gtk::IconSet(refDesat));

        // add all icons to the factory so that they can be used later on.
        iconFactory->add(*pLightenID, *pLighten);
        iconFactory->add(*pDarkenID, *pDarken);
        iconFactory->add(*pSatID, *pSat);
        iconFactory->add(*pDesatID, *pDesat);


        /* Create the main menu */
        m_refActionGroup = Gtk::ActionGroup::create();
        debug("Created ActionGroup");

        /* The File Menu */
        m_refActionGroup->add(Gtk::Action::create("MenuFile", _("_File")));
        m_refActionGroup->add(Gtk::Action::create("New", Gtk::Stock::NEW),
                sigc::mem_fun(*this, &MainWindow::on_action_file_new) );
        m_refActionGroup->add(Gtk::Action::create("SaveScheme", _("_Save Scheme")),
                sigc::mem_fun(*this, &MainWindow::on_action_file_save));
        m_refActionGroup->add(Gtk::Action::create("Quit", Gtk::Stock::QUIT),
                sigc::mem_fun(*this, &MainWindow::on_action_file_quit));   

        /* The Edit Menu */
        m_refActionGroup->add(Gtk::Action::create("MenuEdit", _("_Edit")));
        m_refActionGroup->add(Gtk::Action::create("Preferences", Gtk::Stock::PREFERENCES),
                sigc::mem_fun(*this, &MainWindow::on_action_edit_prefs));   

        /* The Help menu */
        m_refActionGroup->add(Gtk::Action::create("MenuHelp", _("_Help")));
        m_refActionGroup->add(Gtk::Action::create("Contents", Gtk::Stock::HELP),
                sigc::mem_fun(*this, &MainWindow::on_action_help_contents));   
        // Gtk::Stock::About requires gtkmm 2.6
        m_refActionGroup->add(Gtk::Action::create("About", Gtk::Stock::ABOUT),
                sigc::mem_fun(*this, &MainWindow::on_action_help_about));   


        // History Navigation
        m_refActionGroup->add(Gtk::Action::create("HistoryBack", Gtk::Stock::GO_BACK),
                Gtk::AccelKey("<alt>Left"),
                sigc::mem_fun(*this, &MainWindow::on_action_history_back));
        m_refActionGroup->get_action("HistoryBack")->set_sensitive(false);
        m_refActionGroup->get_action("HistoryBack")->property_is_important() = true;
        m_refActionGroup->add(Gtk::Action::create("HistoryFwd", Gtk::Stock::GO_FORWARD),
                Gtk::AccelKey("<alt>Right"),
                sigc::mem_fun(*this, &MainWindow::on_action_history_fwd));
        m_refActionGroup->get_action("HistoryFwd")->set_sensitive(false);

        // Bookmark Actions
        m_refActionGroup->add(Gtk::Action::create("MenuBookmarks", _("Fa_vorites")));
        m_refActionGroup->add(Gtk::Action::create("AddBookmark", Gtk::Stock::ADD,
                    _("Add to Favorites"), _("Add the current color to favorites")),
                Gtk::AccelKey("<control>D"),
                sigc::mem_fun(*this, &MainWindow::on_action_add_bookmark));
        m_refActionGroup->add(Gtk::Action::create("RemoveBookmark", Gtk::Stock::REMOVE,
                    _("Remove Selected"), _("Remove the selected color from your favorites")),
                Gtk::AccelKey("<control>X"),
                sigc::mem_fun(*this, &MainWindow::on_action_remove_bookmark));
        m_refActionGroup->get_action("RemoveBookmark")->set_sensitive(false);
        m_refActionGroup->add(Gtk::Action::create("ClearBookmarks", Gtk::Stock::CLEAR, 
                    _("_Clear All"), _("Clear the list of favorite colors")),
                    sigc::mem_fun(*this, &MainWindow::on_action_clear_bookmarks));
        m_refActionGroup->get_action("ClearBookmarks")->set_sensitive(false);

        // Actions for lightness and saturation
        m_refActionGroup->add(Gtk::Action::create("LightenScheme", *pLightenID,
                    _("_Lighten Scheme"), _("Increase the brightness")),
                Gtk::AccelKey("KP_Add"),
                sigc::mem_fun(*this, &MainWindow::on_action_lighten_scheme));
        m_refActionGroup->add(Gtk::Action::create("DarkenScheme", *pDarkenID,
                    _("_Darken Scheme"), _("Decrease the brightness")),
                Gtk::AccelKey("KP_Subtract"),
                sigc::mem_fun(*this, &MainWindow::on_action_darken_scheme));
        m_refActionGroup->add(Gtk::Action::create("SaturateScheme", *pSatID,
                    _("_Saturate Scheme"), _("Increase the saturation")),
                Gtk::AccelKey("<control>KP_Add"),
                sigc::mem_fun(*this, &MainWindow::on_action_saturate_scheme));
        m_refActionGroup->add(Gtk::Action::create("DesaturateScheme", *pDesatID,
                    _("D_esaturate Scheme"), _("Decrease the saturation")),
                Gtk::AccelKey("<control>KP_Subtract"),
                sigc::mem_fun(*this, &MainWindow::on_action_desaturate_scheme));

        m_refActionGroup->add(Gtk::Action::create("Randomize", Gtk::Stock::REFRESH,
                    _("_Random"), _("Generate a random color")),
                Gtk::AccelKey("<control>R"),
                sigc::mem_fun(*this, &MainWindow::on_action_randomize));
        m_refActionGroup->get_action("Randomize")->property_is_important() = true;

        // Actions for context menu on colorswatches
        m_refActionGroup->add(Gtk::Action::create("UseColor", "_Use"),
                sigc::mem_fun(*this, &MainWindow::on_action_use_color));
        }

        catch (Glib::Error& ex)
        {
            std::cerr << "Failed to create stock icons" << std::endl
                << ex.what() << std::endl;
            throw ex;
        }
    }


    // Actions must be initialized first
    void MainWindow::init_ui(void)
    {
        m_refUIManager = Gtk::UIManager::create();
        m_refUIManager->insert_action_group(m_refActionGroup);
        add_accel_group(m_refUIManager->get_accel_group());

        try
        {
            m_refUIManager->add_ui_from_file(COLORSCHEME_UIDIR "/colorscheme.ui");
            debug("added UI");
        }
        catch(const Glib::Error& ex)
        {
            std::cerr << "building menus failed: " <<  std::endl 
                << ex.what() << std::endl;
            throw ex;
        }

    }


    void MainWindow::on_show(void)
    {
        Gtk::Window::on_show();
        on_schemetype_changed();
        ColorPtr start = Color::create("#9DFF00");
        set_color(start);
    }


    /* File Menu Actions */
    void MainWindow::on_action_file_new(void)
    {
        debug("file->new");
        static gint i = 1;
        std::vector<Glib::ustring> palettes;
        palettes.push_back(COLORSCHEME_PALETTEDIR "/Tango-Palette.gpl");
        palettes.push_back(COLORSCHEME_PALETTEDIR "/Web.gpl");
        palettes.push_back(COLORSCHEME_PALETTEDIR "/Visibone.gpl");
        palettes.push_back(COLORSCHEME_PALETTEDIR "/Ximian-Palette.gpl");
        m_PaletteView->set_from_file(palettes[i % palettes.size()]);
        i++;
    }


    void MainWindow::on_action_file_save(void)
    {
    }


    void MainWindow::on_action_file_quit(void)
    {
        hide();
    }


    /* Edit Menu Actions */
    void MainWindow::on_action_edit_prefs(void)
    {
    }


    /* Help Menu Actions */
    void MainWindow::on_action_help_contents(void)
    {
    }


    void MainWindow::on_action_help_about(void)
    {
        m_pAbout->show();
    }

    void MainWindow::on_color_changed(void)
    {
        ColorPtr clr = Color::create(m_pColorButton->get_color());
        m_pSchemeBox->set_color(clr);
        m_pHistory->add(clr->get_hexstring());
        //debug(*m_pHistory);
        m_refActionGroup->get_action("HistoryBack")->set_sensitive(m_pHistory->has_back());
        m_refActionGroup->get_action("HistoryFwd")->set_sensitive(m_pHistory->has_forward());

        // check if we're at limits
        Glib::RefPtr<Gtk::Action> action =
            m_refActionGroup->get_action("LightenScheme");
        if (clr->get_value() == maxSvValue)
        {
            // disable lighten button
            action->set_sensitive(false);
        }
        else
        {
            action->set_sensitive();
        }

        action = m_refActionGroup->get_action("DarkenScheme");
        if (clr->get_value() <= minColorValue + 5)
        {
            // disable darken button
            action->set_sensitive(false);
        }
        else
        {
            action->set_sensitive();
        }

        action = m_refActionGroup->get_action("SaturateScheme");
        if (clr->get_saturation() == maxSvValue)
        {
            // disable saturate button
            action->set_sensitive(false);
        }
        else
        {
            action->set_sensitive();
        }

        action = m_refActionGroup->get_action("DesaturateScheme");
        if (clr->get_saturation() <= minColorValue + 5)
        {
            // disable desaturate button
            action->set_sensitive(false);
        }
        else
        {
            action->set_sensitive();
        }
        debug("Color was changed!");
        ColorPtr ptr = m_pBookmarkList->get_color();
        if (ptr && (*ptr != *clr))
        {
            m_pBookmarkList->get_selection()->unselect_all();
        }
    }


    void MainWindow::on_schemetype_changed(void)
    {
        tSchemeType type = m_pSchemeSelector->get_scheme_type();
        m_pSchemeBox->set_scheme_type(type);

        debug("Scheme type is ", type);
    }


    void MainWindow::on_menu_file_popup_generic(void)
    {
        std::cout << "mainwindow popup handler" << std::endl;
    }

    void MainWindow::on_action_history_back(void)
    {
        debug("Going Back");
        ColorPtr c = Color::create(m_pHistory->go_back());
        set_color(c);
    }

    void MainWindow::on_action_history_fwd(void)
    {
        debug("Going Forward");
        ColorPtr c = Color::create(m_pHistory->go_forward());
        set_color(c);
    }

    void MainWindow::on_action_lighten_scheme(void)
    {
        ColorPtr clr = Color::create(m_pColorButton->get_color());
        // bump value by 5%
        clr->set_value(static_cast<int>(0.05 * maxSvValue + clr->get_value()));
        set_color(clr);
    }

    void MainWindow::on_action_darken_scheme(void)
    {
        ColorPtr clr = Color::create(m_pColorButton->get_color());
        // decrease value by 5%
        clr->set_value(static_cast<int>(clr->get_value() - 0.05 * maxSvValue));
        set_color(clr);
    }

    void MainWindow::on_action_saturate_scheme(void)
    {
        ColorPtr clr = Color::create(m_pColorButton->get_color());
        // increase saturation by 5%
        clr->set_saturation(static_cast<int>(clr->get_saturation() + 0.05 * maxSvValue));
        set_color(clr);
    }

    void MainWindow::on_action_desaturate_scheme(void)
    {
        ColorPtr clr = Color::create(m_pColorButton->get_color());
        // decrease value by 5%
        clr->set_saturation(static_cast<int>(clr->get_saturation() - 0.05 * maxSvValue));
        set_color(clr);
    }


    void MainWindow::on_action_randomize(void)
    {
        gint r, g, b;
        r = rand_range(maxRgbValue);
        g = rand_range(maxRgbValue);
        b = rand_range(maxRgbValue);
        ColorPtr clr = Color::create(r, g, b);
        set_color(clr);
    }

    gint MainWindow::rand_range(gint range)
    {
        gint ret = static_cast<gint>(
                static_cast<double>(rand()) / static_cast<double>(RAND_MAX) *
                static_cast<double>(range));
        debug("Random value: ", ret);
        return ret;
    }


    void MainWindow::on_action_use_color(void)
    {}

    void MainWindow::on_bookmarks_selection_changed(void)
    {
        debug("Bookmarks changed");
        if (m_pBookmarkList->get_selection()->count_selected_rows())
        {
            debug(" -> at least one selected");
            set_color(m_pBookmarkList->get_color());
        }
        update_bookmark_actions();
    }


    void MainWindow::on_schemebox_color_selected(tHexString hex)
    {
        ColorPtr c = Color::create(hex);
        set_color(c);
    }


    void MainWindow::on_action_add_bookmark(void)
    {
        ColorPtr clr = Color::create(m_pColorButton->get_color());
        m_pBookmarkList->add(clr);
        update_bookmark_actions();
    }

    void MainWindow::on_action_remove_bookmark(void)
    {
        m_pBookmarkList->remove_selected();
        update_bookmark_actions();
    }

    void MainWindow::on_action_clear_bookmarks(void)
    {
        debug("Clearing Bookmarks");
        m_pBookmarkList->clear();
        update_bookmark_actions();
    }

    void MainWindow::update_bookmark_actions(void)
    {
        m_refActionGroup->get_action("RemoveBookmark")->set_sensitive(!m_pBookmarkList->empty()
                && m_pBookmarkList->count_selected());
        m_refActionGroup->get_action("ClearBookmarks")->set_sensitive(!m_pBookmarkList->empty());
    }

    void MainWindow::set_color(ColorPtr c)
    {
        m_pColorButton->set_color(c->gdk());
        on_color_changed();
    }


} // namespace gcs
