/*******************************************************************************
 *  PROJECT: GNOME Colorscheme
 *
 *  AUTHOR: Jonathon Jongsma
 *
 *  Copyright (c) 2005 Jonathon Jongsma
 *
 *  License:
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the 
 *    Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 *    Boston, MA  02111-1307  USA
 *
 *******************************************************************************/

#include <iostream>
#include <iomanip>
#include <fstream>
#include <libgnomevfsmm/handle.h>
#include <libgnomevfsmm/init.h>

#include "gcs-bookmarklist.h"
#include "gcs-cellrendererswatch.h"
#include "../gcs-debug.h"
#include "../gcs-i18n.h"

namespace gcs
{
    namespace Widgets
    {
        BookmarkList::BookmarkList(Glib::RefPtr<Gnome::Vfs::Uri> fUri) :
            Palette(fUri),
            m_file(fUri),
            m_refListStore(Gtk::ListStore::create(m_columns))
        {
            set_model(m_refListStore);

            // display column headers
            set_headers_visible(true);

            // Create a instance of a custom CellRenderer for drawing color
            // swatches
            CellRendererSwatch *const pRenderer = Gtk::manage(new
                    CellRendererSwatch);
            // Create a new Column named "Favorites"
            Gtk::TreeViewColumn *const pColumn = Gtk::manage(new
                    Gtk::TreeViewColumn(_("Favorites")));
            // Pack the custom swatch renderer into the column
            pColumn->pack_start(*pRenderer, false);
            // associate the 'color' property of the custom renderer with the
            // value of m_columns.m_colColor so that the cell renderer can color
            // each swatch according to the bookmarked item
            pColumn->add_attribute(pRenderer->property_color(),
                    m_columns.m_colColor);
            // add a the model text column to the TreeView column -- This packs
            // both the swatch and the hexstring into the same treeview column
            pColumn->pack_start(m_columns.m_colText);
            // append the column to the TreeView
            append_column(*pColumn);
            set_reorderable();

            // add the bookmarks that were on disk to the UI
            for (std::vector<ColorPtr>::iterator iter = Palette::m_colors.begin();
                    iter != Palette::m_colors.end(); ++iter)
            {
                add_(*iter);
            }
        }


        void BookmarkList::add(ColorPtr clr)
        {
            // Assume the the color we want to add is not already in the list
            bool exists = false;
            // Loop through all the items in the list 
            for (Gtk::TreeModel::iterator iter = m_refListStore->children().begin();
                    iter != m_refListStore->children().end();
                    ++iter)
            {
                // check if the current item in the list matches the 
                // color value to be added
                // NOTE: need to use get_value() here instead of operator[]
                if (*(*iter).get_value(m_columns.m_colColor) == *clr)
                {
                    exists = true;
                    get_selection()->select(iter);
                    break;
                }
            }
            if (!exists)
            {
                add_(clr);
                // save the bookmarks to disk
                save_to_disk();
            }
        }


        void BookmarkList::add_(ColorPtr clr)
        {
                // add a new row to the end
                Gtk::TreeModel::iterator iter = m_refListStore->append();
                Gtk::TreeModel::Row row = *iter;
                // set the columns in the list store
                row[m_columns.m_colColor] = clr;
                row[m_columns.m_colText] = clr->get_hexstring();
                // select the newly added row
                Glib::RefPtr<Gtk::TreeView::Selection> sel = get_selection();
                sel->select(*iter);
        }

        // returns the hexstring of the currently selected row
        ColorPtr BookmarkList::get_color(void)
        {
            ColorPtr ptr;
            Glib::RefPtr<Gtk::TreeSelection> refSel = get_selection();
            Gtk::TreeModel::iterator iter = refSel->get_selected();
            if (iter)
            {
                Gtk::TreeModel::Row row = *iter;
                // need to use get_value() here instead of operator[]
                ptr = row.get_value(m_columns.m_colColor);
            }
            return ptr;
        }


        // clear out the bookmarks list
        void BookmarkList::clear(void)
        {
            get_selection()->unselect_all();
            m_refListStore->clear();
            save_to_disk();
        }


        // check if the list is empty
        bool BookmarkList::empty(void)
        {
            return m_refListStore->children().empty();
        }


        void BookmarkList::remove_selected(void)
        {
            Glib::RefPtr<Gtk::TreeSelection> sel = get_selection();
            Gtk::TreeModel::iterator iter = sel->get_selected();
            sel->unselect(iter);
            m_refListStore->erase(iter);
            save_to_disk();
        }


        gint BookmarkList::count_selected(void)
        {
            return get_selection()->count_selected_rows();
        }


        void BookmarkList::load_from_file(void)
        {
            if (m_file && m_file->uri_exists())
            {
                std::ifstream fav_file(m_file->get_path().c_str());
                if(fav_file.is_open())
                {
                    // parse it
                }
                else
                {
                    std::cerr << "*** Couldn't open bookmarks file for reading" << std::endl;
                }
            }
        }


        void BookmarkList::save_to_disk(void)
        {
            if (m_file)
            {
                if (!m_file->get_parent()->uri_exists())
                {
                    // create the directory for holding colorscheme data
                    // must have execute bit for creating files inside the directory
                    Gnome::Vfs::Handle::make_directory(m_file->get_parent(), 0755);
                }
                std::ofstream fav_file(m_file->get_path().c_str());
                if (fav_file.is_open())
                {
                    Glib::ustring name = Glib::get_real_name();
                    if (!name.size())
                    {
                        name = Glib::get_user_name();
                    }
                    name = Glib::ustring("Bookmarks for ") + name;
                    // write header
                    fav_file << "GIMP Palette" << std::endl;
                    fav_file << "Name: " << name << std::endl;
                    if (Palette::m_columns)
                    {
                        fav_file << "Columns: " << Palette::m_columns << std::endl;
                    }
                    fav_file << "#" << std::endl;
                    fav_file << "# generated by GNOME Colorscheme" << std::endl;
                    fav_file << "#" << std::endl;
                    // write the colors
                    for (Gtk::TreeModel::iterator iter = m_refListStore->children().begin();
                            iter != m_refListStore->children().end();
                            ++iter)
                    {
                        ColorPtr p = (*iter).get_value(m_columns.m_colColor);
                        fav_file << std::setw(3) << p->get_red() << " "
                            << std::setw(3) << p->get_green() << " "
                            << std::setw(3) << p->get_blue() << " "
                            << p->get_hexstring() << std::endl;
                    }
                }
                else
                {
                    std::cerr << "*** Error opening bookmarks for writing" << std::endl;
                }
            }
        }

    }   // namespace Widgets 

}   // namespace gcs
