/*******************************************************************************
 *  PROJECT: GNOME Colorscheme
 *
 *  AUTHOR: Jonathon Jongsma
 *
 *  Copyright (c) 2005 Jonathon Jongsma
 *
 *  License:
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the 
 *    Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 *    Boston, MA  02111-1307  USA
 *
 *******************************************************************************/

#include <iostream>
#include <sstream>

#include <gtkmm/actiongroup.h>
#include <gtkmm/stock.h>
#include <gtkmm/clipboard.h>
#include <gdkmm/gc.h>
#include <pangomm/layout.h>
#include <glibmm/ustring.h>

#include "gcs-colorswatch.h"
#include "../gcs-debug.h"
#include "../gcs-i18n.h"


namespace gcs
{
    namespace Widgets
    {
        ColorPtr ColorSwatch::m_white = gcs::Color::create("#FFFFFF");
        ColorPtr ColorSwatch::m_black = gcs::Color::create("#000000");

        ColorSwatch::ColorSwatch(ColorPtr bg)
            :
            m_background(bg),
            m_borderWidth(1), m_minSize(12),
            m_swatchPadding(6)
            {
                // get_window() would return 0 because the Gdk::Window has not yet been
                // realized So we can only allocate the colors here - the rest will
                // happen in on_realize().
                m_colormap = get_default_colormap();
                m_colormap->alloc_color(m_black->gdk());
                m_colormap->alloc_color(m_white->gdk());
                alloc_bg();
                debug("ColorSwatch Created!");
            }


        ColorSwatch::ColorSwatch(const ColorSwatch& c)
            : m_background(c.m_background)
        {
            m_borderGC = c.m_borderGC;
            m_colormap = c.m_colormap;
            debug("ColorSwatch Copied!");
        }


        ColorSwatch::~ColorSwatch(void)
        {
            //delete &m_background;
            debug("ColorSwatch Destroyed!");
        }


        void ColorSwatch::on_realize(void)
        {
            debug("ColorSwatch Realized!");
            // We need to call the base on_realize()
            Gtk::DrawingArea::on_realize();

            // Now we can allocate any additional resources we need
            Glib::RefPtr<Gdk::Window> window = get_window();
            // set the size of the swatch.
            set_size_request(m_minSize, m_minSize);
            m_borderGC = Gdk::GC::create(window);
            m_borderGC->set_line_attributes(m_borderWidth, Gdk::LINE_SOLID,
                    Gdk::CAP_BUTT, Gdk::JOIN_MITER);
            m_borderGC->set_foreground(m_black->gdk());
            add_events(Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK);
        }


        bool ColorSwatch::on_expose_event(GdkEventExpose *e)
        {
            redraw();
            return true;
        }


        void ColorSwatch::redraw(void)
        {
            Glib::RefPtr<Gdk::Window> win = get_window();
            win->set_background(m_background->gdk());
            win->clear();
            int winWidth, winHeight;
            win->get_size(winWidth, winHeight);
            /* First draw an outline around the edge (in black, i guess) */
            if (m_borderWidth > 0)
            {
                win->draw_rectangle(m_borderGC, false, m_borderWidth/2, m_borderWidth/2,
                        winWidth - m_borderWidth, winHeight - m_borderWidth);
            }
        }


        void ColorSwatch::alloc_bg(void)
        {
            m_colormap->alloc_color(m_background->gdk());
        }


        void ColorSwatch::set_color(ColorPtr bg)
        {
            m_background = bg;
            alloc_bg();
            debug("background_ = ", m_background);
            redraw();
        }


        void ColorSwatch::set_border_width(gint width)
        {
            m_borderWidth = width;
        }


        bool ColorSwatch::on_button_press_event(GdkEventButton *e)
        {
            debug("button pressed: ", e->button);

            //Then do our custom stuff:
            if (e->type == GDK_BUTTON_PRESS)
            {
            }
            return true;
        }


        bool ColorSwatch::on_button_release_event(GdkEventButton *e)
        {
            debug("button released: ", e->button);

            //Then do our custom stuff:
            if (e->type == GDK_BUTTON_RELEASE)
            {
                if (e->button == 1)
                {
                    // User pressed left mouse button
                    m_signal_selected.emit();
                    debug("In on_button_press_event; emitted signal_selected()");
                }
            }
        return true;
        }

    } // namespace Widgets
} // namespace gcs
